#
#!/bin/ksh
#!/bin/bash
#
#  File           : run.sh
#  Module         : MRL 
#  Purpose        : convenience script for MRL processing
#                   This shell script executes gentrn, gendata, genprint, 
#                   and genarc for the AFP, PCL, PST, and XER print types.
#  Synopsis       : run.sh PrintType ProcessType
#  Pre-requisites : 
#
###################################################################


thisscript=run.sh

function usage
{
   vflag=0
   hflag=0
   aflag=0
   bflag=0
   pflag=0
   hasflags=0
   invaliddirs=0
   doproceed=0
   doneclear=0
   testfromdir=0
   testfromdirset="0"
   testtodir=0
   testtodirset="0"

   printf '  Usage: %s [ -vVhHuUfFcCaAwW -e PATH PrintType ProcessType ]\n' "$thisscript"
   printf '          options: -v       (turn verbose on)\n'
   printf '                   -h       (print this screen)\n'
   printf '                   -u       (up2low entire tree, e.g. RP 10.0 & 10.1)\n'
   printf '                   -c       (convert files toUNIX text and "\\" to "/", \n'
   printf '                                e.g. RP 11.4 and up)\n'
   printf '                   -e PATH  (EXE and DSO paths)\n'
   printf '          PrintType:\n'
   printf '                    p - for PCL (HP Printer Control Language)\n'
   printf '          ProcessType:\n'
   printf '                    s - Single Step Processing (gentrn, gendata, and genprint are combined)\n'
   printf '                    m - Multi-Step Processing (gentrn, gendata, and genprint are run seperately)\n'
   printf '   Example: qarun.sh\n'
   printf '              (causes interactive menu system to prompt user for options)\n'
   printf '\n'
   printf '   Example: run.sh p m\n'
   printf '              (generates data for PCL printer using the multi-step process)\n'
   printf '\n'
   printf '   Example: qarun.sh p s\n'
   printf '              (generates data for PCL printer using the single-step process\n'
   printf '               without interaction)\n'
   printf '\n'
   printf 'Nothing processsed, rerun with correct arguments!\n'
   OPTIND=1
   exit 1

}

function make_a_directory
{
   if [[ ".$vflag" = ".1" ]] ; then printf '%s: calling (mkdir %s)\n' "$thisscript" "$TEMP_DIRNAME"; fi
   if [[ ! -d "$TEMP_DIRNAME" ]] ; then mkdir $TEMP_DIRNAME; fi
   if [[ ".$?" != ".0" ]] ; then printf '%s: ERROR: (mkdir %s) FAILED!, PROCESS STOPPED!!!!\n' "$thisscript" "$TEMP_DIRNAME"; exit 1; fi
   printf '.'
}

function copy_files
{
   if [[ "x$RECURSIVE_COPY" = "x1" ]] ; then
       if [[ ".$vflag" = ".1" ]] ; then printf '%s: calling (cp -r %s %s)\n' "$thisscript" "$COPY_FROM_FILE" "$COPY_TO_FILE"; fi
       cp -r $COPY_FROM_FILE $COPY_TO_FILE 
       if [[ ".$?" != ".0" ]] ; then printf '%s: ERROR: (cp -r %s %s) FAILED!, PROCESS STOPPED!!!!\n' "$thisscript" "$COPY_FROM_FILE" "$COPY_TO_FILE"; exit 1; fi
   else 
       if [[ ".$vflag" = ".1" ]] ; then printf '%s: calling (cp %s %s)\n' "$thisscript" "$COPY_FROM_FILE" "$COPY_TO_FILE"; fi
       cp $COPY_FROM_FILE $COPY_TO_FILE 
       if [[ ".$?" != ".0" ]] ; then printf '%s: ERROR: (cp %s %s) FAILED!, PROCESS STOPPED!!!!\n' "$thisscript" "$COPY_FROM_FILE" "$COPY_TO_FILE"; exit 1; fi
   fi
   printf '.' 
}

function remove_files
{
   if [[ "x$RECURSIVE_REMOVE" = "x1" ]] ; then
      if [[ ".$vflag" = ".1" ]] ; then printf '%s: calling (rm -r %s)\n' "$thisscript" "$REMOVE_THESE_FILES"; fi
      rm -r $REMOVE_THESE_FILES
      # if [[ ".$?" != ".0" ]] ; then printf '%s: ERROR: (rm -r %s) FAILED!, PROCESS STOPPED!!!!\n' "$thisscript" "$REMOVE_THESE_FILES"; exit 1; fi
   else 
      if [[ ".$vflag" = ".1" ]] ; then printf '%s: calling (rm %s)\n' "$thisscript" "$REMOVE_THESE_FILES"; fi
      rm $REMOVE_THESE_FILES
      # if [[ ".$?" != ".0" ]] ; then printf '%s: ERROR: (rm %s) FAILED!, PROCESS STOPPED!!!!\n' "$thisscript" "$REMOVE_THESE_FILES"; exit 1; fi
   fi
   printf '.' 
}

function environment_setup
{
  printf '\n%s: SET ENVIRONMENT VARIABLES =============================================\n' "$thisscript" 
  if [[ ".$vflag" = ".1" ]] ; then printf '%s: running (environment_setup on %s)\n' "$thisscript" "$OS" ; fi

  CURRENTPWD=`pwd`

  printf '%s: Pre-pending <%s> to Path and Library Path\n' "$thisscript" "$environmentval" 
 
  
  #PATH=~/rp/bin:$PATH
  PATH=$environmentval:$PATH
  export PATH

  gentrnexe=$environmentval"/gentrn"
  if ( ( [[ "x$environmentval" != "x" ]] ) && ( [[ -d "$environmentval" ]] ) && ( [[ -x "$gentrnexe" ]] ) ) ; then
     if [[ ".$vflag" = ".1" ]] ; then printf '%s: set PATH to %s (environment_setup on %s)\n' "$thisscript" "$OS" "$PATH" ; fi
  else
     printf 'ERROR: %s is an invalid directory for Documaker RP!\n' "$environmentval"
     exit 1
  fi

  case $OS in
    HPUX* )
        #SHLIB_PATH=~/rp/bin:$SHLIB_PATH
        SHLIB_PATH=$environmentval:$SHLIB_PATH
        export SHLIB_PATH
        ;;
    AIX* )
        #LIBPATH=~/rp/bin:$LIBPATH
        LIBPATH=$environmentval:$LIBPATH
        export LIBPATH
        ;;
    SPARC* )
        #LD_LIBRARY_PATH=~/rp/bin:$LD_LIBRARY_PATH
        LD_LIBRARY_PATH=$environmentval:$LD_LIBRARY_PATH
        export LD_LIBRARY_PATH
        if [[ ".$vflag" = ".1" ]] ; then printf '%s: set LD_LIBRARY_PATH to %s (environment_setup on %s)\n' "$thisscript" "$OS" "$LD_LIBRARY_PATH" ; fi
        ;;
    LINUX* )
        #LD_LIBRARY_PATH=~/rp/bin:$LD_LIBRARY_PATH
        LD_LIBRARY_PATH=$environmentval:$LD_LIBRARY_PATH
        export LD_LIBRARY_PATH;;
    *)
        printf '%s: ERROR: (environment_setup can not set PATH and Library PATH) FAILED!, PROCESS STOPPED!!!!\n' "$thisscript" 
        exit 1
        ;;
  esac



  if [[ ".$vflag" = ".1" ]] ; then printf '%s: completed (up2low -R %s).\n' "$thisscript" "$CURRENTPWD" ; fi
}


function pcl_setup
{
  printf '\n%s: PCL STARTING ==========================================================\n' "$thisscript"
  if [[ ".$vflag" = ".1" ]] ; then printf '%s: running (pcl_setup)\n' "$thisscript" ; fi

  #cp forms/pclovl/*.ovl forms/
  #if [[ ".$?" != ".0" ]] ; then printf '%s: ERROR: (cp forms/pclovl/*.ovl forms/) FAILED!, PROCESS STOPPED!!!!\n' "$thisscript" ; exit 1; fi

  cp deflib/afgjob.$proctype deflib/afgjob.jdt
  if [[ ".$?" != ".0" ]] ; then printf '%s: ERROR: (cp deflib/afgjob.%s deflib/afgjob.jdt) FAILED!, PROCESS STOPPED!!!!\n' "$thisscript" "$proctype" ; exit 1; fi

 cp pclini/fsisys.$proctype ./fsisys.ini
  if [[ ".$?" != ".0" ]] ; then printf '%s: ERROR: (cp pclini/fsisys.%s ./fsisys.ini) FAILED!, PROCESS STOPPED!!!!\n' "$thisscript" "$proctype" ; exit 1; fi

  if [[ ".$vflag" = ".1" ]] ; then printf '%s: completed (pcl_setup).\n' "$thisscript" ; fi
}

function run_multistep
{
  printf '\n%s: RUNNING MULTI-STEP ====================================================\n' "$thisscript" 
  if [[ ".$vflag" = ".1" ]] ; then printf '%s: running (run_multistep)\n' "$thisscript" ; fi
  gentrn
  gendata
  genprint
  genwip
  genarc
  if [[ ".$vflag" = ".1" ]] ; then printf '%s: completed (run_multistep).\n' "$thisscript" ; fi
}

function run_singlestep
{
  printf '\n%s: RUNNING SINGLE-STEP ===================================================\n' "$thisscript" 
  if [[ ".$vflag" = ".1" ]] ; then printf '%s: running (run_singlestep)\n' "$thisscript" ; fi
  gendata
  genwip
  genarc
  if [[ ".$vflag" = ".1" ]] ; then printf '%s: completed (run_singlestep).\n' "$thisscript" ; fi
}


#---------------------------------------------------------------
# Determine OS type
#---------------------------------------------------------------
OSTYPE=`uname -s`
MACHINEMODULE=`uname -m`
OSRELEASE=`uname -r`
OSVERSION=`uname -v`
MACHINENAME=`uname -n`
case $OSTYPE in
    *HP*UX | *hp*ux )
        AWK=awk
        OS=HPUX;;
    *AIX* | *aix* )
        AWK=awk
        OS=AIX ;;
    *Sun* | *sun* | *SUN* )
        AWK=/usr/xpg4/bin/awk
        case $MACHINEMODULE in
            *4u* | *4U* )
                OS=SPARC ;
                OS64=SPARC64 ;;
            *)
                OS=SPARC ;;
        esac ;;
    *Linux* | *linux* | *LINUX* )
        AWK=awk
        OS=LINUX ;;
    *)
        AWK=awk
        OS='???' ;;
esac
export AWK OS


vflag=0
hflag=0
aflag=0
bflag=0
pflag=0
cflag=0
fflag=0
uflag=0
hasflags=0
invaliddirs=0
doproceed=0
doneclear=0
testfromdir=0
testfromdirset="0"
testtodir=0
testtodirset="0"
dorepeat=0
dorecursiveproc=0
fap2cfa_set=
up2low_set=
#printf '$*'
#printf '$OPTIND'
OPTIND=1
while getopts fFcCuUaAwWhHvV\?e:E: name
do
    case $name in
       p | P ) pflag=1
               pval="$OPTARG"
               hasflags=1;;
     esac
done

if [[ "x$hflag" = "x1" ]] ; then
   vflag=0
   hflag=0
   hasflags=0
   invaliddirs=0
   usage
fi


if [ ! -z "$hasflags" ] ; then
   shift $(($OPTIND - 1))
   #shift
   printf "Arguments are: %s\n" "$*"
fi


dorepeat=1

if [[ "x$1" = "x" ]] ; then
#   usage
    noargs=1
fi
#else

#################################################
# Turn these on for run.sh sent to customers
# installation will build this from this base
# do not edit lines that start with #INSTALL_ !!
#INSTALL_environmentval=/home/docc/rel102/bin
#INSTALL_environment_setup
#INSTALL_environment_set=0
#INSTALL_up2low_set=0
#INSTALL_unixconv_set=0
#INSTALL_skip_genwip=1
#INSTALL_skip_genarc=1

###############################################################
# Retrieve any needed information, Print Type, Process Type 
###############################################################


case $1 in
     p | P ) prttype=PCL
             prttypeset=1;;
         * )
             prttype=
             prttypeset=0
             #printf 'ERROR: %s is an invalid Print Type selection.\n' "$1"
             #usage
esac

case $2 in
     s | S ) proctype=s
             proctypeset=1;;
     m | M ) proctype=m
             proctypeset=1;;
         * )
             proctype=
             proctypeset=0
             #printf 'ERROR: %s is an invalid Process Type selection.\n' "$2"
             #usage
esac

#-------------------------------------------------
# Query about ENVIRONMENT setup
#-------------------------------------------------
    if ( ( [[ ".$environment_set" = "." ]] ) && ( ( [[ ".$prttypeset" != ".1" ]] ) || ( [[ ".$proctypeset" != ".1" ]] ) ) ) ; then
       if [[ ".$doneclear" = ".0" ]] ; then clear ; doneclear=1 ; fi
       environment_set=
       while ( [[ ".$environment_set" = "." ]] ) ; do
         printf '\aWhat directory contains the DSOs and EXEs? \n'
         printf ' press enter key for default of %s\n' "~/rp/bin"
         printf '>'
         read environmentval
         if ( [[ "x$environmentval" = "x" ]] ) ; then environmentval=~/rp/bin ; fi
         gentrnexe=$environmentval"/gentrn"
         if ( ( [[ "x$environmentval" != "x" ]] ) && ( [[ -d "$environmentval" ]] ) && ( [[ -x "$gentrnexe" ]] ) ) ; then
            environment_set=1
            environment_setup
         else
            environment_set=
            printf '%s: ERROR: (invalid directory <%s> for environment) FAILED!, PROCESS STOPPED!!!!\n' "$thisscript" "$environmentval"
            exit 1
         fi
       done
       printf '\n'
    else 
       if ( [[ ".$environment_set" = ".1" ]] ) ; then
         gentrnexe=$environmentval"/gentrn"
         if ( ( [[ "x$environmentval" != "x" ]] ) && ( [[ -d "$environmentval" ]] ) && ( [[ -x "$gentrnexe" ]] ) ) ; then
             environment_setup
         else
            printf 'ERROR: %s is an invalid directory!\n' "$environmentval"
            exit 1
         fi
       fi
    fi


#-------------------------------------------------
# Query about UP2LOW
#-------------------------------------------------
    if ( ( [[ ".$up2low_set" = "." ]] ) && ( ( [[ ".$prttypeset" != ".1" ]] ) || ( [[ ".$proctypeset" != ".1" ]] ) ) ) ; then
       if [[ ".$doneclear" = ".0" ]] ; then clear ; doneclear=1 ; fi
       up2low_set=
       while ( [[ ".$up2low_set" = "." ]] ) ; do
         printf '\aDo you want to convert any UPPERcase letters in \n'
         printf 'file names to lowercase if they exist?\n'
         printf '#{y or n}>'
         read up2lowval
         case $up2lowval in
            y|Y )
                    up2low_process
                    up2low_set=1;;
            n|N )
                    up2low_set=0;;
            *)
                    up2low_set=
                    printf 'ERROR: %s is an invalid selection\n' "$up2lowval";;
         esac
       done
       printf '\n'
    else 
       if ( [[ ".$up2low_set" = ".1" ]] ) ; then
          up2low_process
       fi
    fi


#-------------------------------------------------
# Resolve Print Type
#-------------------------------------------------
    if [[ ".$prttypeset" != ".1" ]] ; then
       if [[ ".$doneclear" = ".0" ]] ; then clear ; doneclear=1 ; fi
       while ( [[ ".$prttype" = "." ]] ) ; do
         printf '\aPlease select a Print Type:\n'
         printf '  p = for PCL (HP Printer Control Language)\n'
            read prtval
         case $prtval in
             p|P )
                    prttype=PCL;;
               *)
                    prttype=
                    printf 'ERROR: %s is an invalid Print Type selection\n' "$prtval";;
         esac
       done
       printf '\n'
    fi

#-------------------------------------------------
# Resolve Process Type
#-------------------------------------------------
    if [[ ".$proctypeset" != ".1" ]] ; then
       if [[ ".$doneclear" = ".0" ]] ; then clear ; doneclear=1 ; fi
       while ( [[ ".$proctype" = "." ]] ) ; do
         printf '\aPlease select a Process Type:\n'
         printf '  s = Single Step Processing (gentrn, gendata, and genprint are combined)\n'
         printf '  m = Multi-Step Processing (gentrn, gendata, and genprint are run seperately)\n'
         printf '#>'
         read procval
         case $procval in
            s|S )
                    proctype=s;;
            m|M )
                    proctype=m;;
            *)
                    proctype=
                    printf 'ERROR: %s is an invalid Process Type selection\n' "$procval";;
         esac
       done
       printf '\n'
    fi



   if [[ ".$vflag" = ".1" ]] ; then printf '%s: select complete (Print Type = %s) (Process Type = %s)\n' "$thisscript" "$prttype" "$proctype" ; fi

#---------------------------------------------------------------
# setup OS directories
#---------------------------------------------------------------
   if [[ "x$bits" != "x64" ]] ; then
     case $OS in
       SPARC )
               STDEXT=spc;
               RDROOT=/home/fap;
               QAROOT=/home/fapqa;
               DSODIR=spcdso;
               SLIBDIR=spclib;
               EXEDIR=spcexe;
               BINDIR=spcbin;
               LIBDIR=spclib;
               SHIPBIN=shipspc;
               SYNC=/usr/local/fap-bin/syncfiles;
               UTILSDIR=/usr/local/fap-bin;
               PDF417DIR=/home/fapr/pdf417;
               TRDSRCDIR=/home/fap/3rdparty;;
       AIX )
               STDEXT=aix;
               RDROOT=/home/fap;
               QAROOT=/home/fapqa;
               DSODIR=aixdso;
               SLIBDIR=aixlib;
               EXEDIR=aixexe;
               BINDIR=aixbin;
               LIBDIR=aixlib;
               SHIPBIN=shipaix;
               SYNC=/usr/local/fap-bin/syncfiles;
               UTILSDIR=/usr/local/fap-bin; 
               PDF417DIR=/home/fapr/pdf417;
               TRDSRCDIR=/home/fap/3rdparty;;
       HPUX )
               STDEXT=hpx;
               RDROOT=/sw/fap;
               QAROOT=/sw/fapqa;
               DSODIR=hpxdso;
               SLIBDIR=hpxlib;
               EXEDIR=hpxexe;
               BINDIR=hpxbin;
               LIBDIR=hpxlib;
               SHIPBIN=shiphpx;
               SYNC=/usr/local/fap-bin/syncfiles;
               UTILSDIR=/usr/local/fap-bin;  
               PDF417DIR=/sw/fapr/pdf417;
               TRDSRCDIR=/home/fap/3rdparty;;
       LINUX )
               STDEXT=lnx;
               RDROOT=/home/fap;
               QAROOT=/home/fapqa;
               DSODIR=lnxdso;
               SLIBDIR=lnxlib;
               EXEDIR=lnxexe;
               BINDIR=lnxbin;
               LIBDIR=lnxlib;
               SHIPBIN=shiplnx;
               SYNC=/usr/local/fap-bin/syncfiles;
               UTILSDIR=/usr/local/fap-bin;  
               PDF417DIR=/home/fapr/pdf417;
               TRDSRCDIR=/home/fap/3rdparty;;
     esac
   else
     case $OS in
       SPARC )
               STDEXT=spc;
               RDROOT=/home/fap;
               QAROOT=/home/fapqa;
               DSODIR=spc64dso;
               SLIBDIR=spc64lib;
               EXEDIR=spc64exe;
               BINDIR=spc64bin;
               LIBDIR=spc64lib;
               SHIPBIN=shipspc64;
               SYNC=/usr/local/fap-bin/syncfiles;
               UTILSDIR=/usr/local/fap-bin;
               PDF417DIR=/home/fapr/pdf417;
               TRDSRCDIR=/home/fap/3rdparty;;
       AIX )
               STDEXT=aix;
               RDROOT=/home/fap;
               QAROOT=/home/fapqa;
               DSODIR=aix64dso;
               SLIBDIR=aix64lib;
               EXEDIR=aix64exe;
               BINDIR=aix64bin;
               LIBDIR=aix64lib;
               SHIPBIN=shipaix64;
               SYNC=/usr/local/fap-bin/syncfiles;
               UTILSDIR=/usr/local/fap-bin;
               PDF417DIR=/home/fapr/pdf417;
               TRDSRCDIR=/home/fap/3rdparty;;
       HPUX )
               STDEXT=hpx;
               RDROOT=/home/fap;
               QAROOT=/sw/fapqa;
               DSODIR=hpx64dso;
               SLIBDIR=hpx64lib;
               EXEDIR=hpx64exe;
               BINDIR=hpx64bin;
               LIBDIR=hpx64lib;
               SHIPBIN=shiphpx64;
               SYNC=/usr/local/fap-bin/syncfiles;
               UTILSDIR=/usr/local/fap-bin;
               PDF417DIR=/sw/fapr/pdf417;
               TRDSRCDIR=/home/fap/3rdparty;;
       LINUX )
               STDEXT=lnx;
               RDROOT=/home/fap;
               QAROOT=/home/fapqa;
               DSODIR=lnx64dso;
               SLIBDIR=lnx64lib;
               EXEDIR=lnx64exe;
               BINDIR=lnx64bin;
               LIBDIR=lnx64lib;
               SHIPBIN=shiplnx64;
               SYNC=/usr/local/fap-bin/syncfiles;
               UTILSDIR=/usr/local/fap-bin;
               PDF417DIR=/home/fapr/pdf417;
               TRDSRCDIR=/home/fap/3rdparty;;
     esac
   fi

   if [[ ".$vflag" = ".1" ]] ; then printf '%s: processing main functions:\n' "$thisscript" ; fi

   if [[ ".$prttype" = ".PCL" ]] ; then 
       if [[ ".$vflag" = ".1" ]] ; then printf '%s: calling (pcl_setup)\n' "$thisscript" ; fi
       pcl_setup
   fi

   if [[ ".$prttype" != ".ALL" ]] ; then
      if [[ ".$proctype" = ".s" ]] ; then
          if [[ ".$vflag" = ".1" ]] ; then printf '%s: calling (run_singlestep)\n' "$thisscript" ; fi
          run_singlestep
      elif [[ ".$proctype" = ".m" ]] ; then 
          if [[ ".$vflag" = ".1" ]] ; then printf '%s: calling (run_multistep)\n' "$thisscript" ; fi
          run_multistep
      fi
   fi